/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************
 *   Maarten de Boer <maarten.deboer@iua.upf.es>, 1999                     *
 *   Music Technology Group                                                *
 *   Institut de l'Audiovisual, Universitat Pompeu Fabra, Barcelona, Spain *
 *   http://www.iua.upf.es/mtg/                                            *
 ***************************************************************************/

#include <FL/Fl.H>
#include "DragValue.H"
#include <FL/fl_draw.H>
#include <stdlib.h>

#ifndef macintosh
	#include <FL/Fl_Pixmap.H>
	#include "mag.xpm"
#endif

#ifdef macintosh
	#include <stdlib.h>
#endif


void DragValue::draw() {
  Fl_Color col = value() ? selection_color() : color();
#if FL_MINOR_VERSION==1
  draw_box((Fl_Boxtype) (down_ ? fl_down(box()) : box()), col);
#else
  draw_box((Fl_Boxtype) (down_ ? down(box()) : box()), col);
#endif
  draw_label();
}        
                                                                              
int DragValue::handle(int event) {
  if (!step()) return 0;
  double v;
  int delta, tmp;
  int mx = Fl::event_x();
  int my = Fl::event_y();
  static int ix, iy,drag;
  switch (event) {
  case FL_PUSH:
    ix = mx;
    iy = my;
    drag = Fl::event_button();
		down_=1;
    handle_push();
		redraw();
    return 1;
  case FL_DRAG:
    delta = Fl::event_x()-ix;
		tmp = Fl::event_y()-iy;		
		if (abs(tmp)>abs(delta)) delta = tmp;
    if (delta > 5) delta -= 5;
    else if (delta < -5) delta += 5;
    else delta = 0;
    switch (drag) {
    case 3: v = increment(previous_value(),delta*100); break;
    case 2: v = increment(previous_value(),delta*10); break;
    default:v = increment(previous_value(),delta); break;
    }
    v = round(v);
    handle_drag(soft()?softclamp(v):clamp(v));;
    return 1;
  case FL_RELEASE:
		down_=0;
    handle_release();
		redraw();
    return 1;
  default:
    return 0;
  }
}

#ifndef macintosh
static Fl_Pixmap p(mag_xpm);
#endif

DragValue::DragValue(int x,int y,int w,int h,const char *l)
: Fl_Valuator(x,y,w,h,l) {
  box(FL_THIN_UP_BOX);
  soft_ = 0;
	down_ = 0;
#ifndef macintosh
	p.label(this);
#endif
	align(FL_ALIGN_INSIDE);
}

