/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************
 *   Maarten de Boer <maarten.deboer@iua.upf.es>, 1999                     *
 *   Music Technology Group                                                *
 *   Institut de l'Audiovisual, Universitat Pompeu Fabra, Barcelona, Spain *
 *   http://www.iua.upf.es/mtg/                                            *
 ***************************************************************************/

#include "SDIFDisplayTRC.H"
#include "SDIFDisplayColors.H"
#include <FL/fl_draw.H>

#include "SDIFDisplayColors.H"

SDIFDisplayTRC::SDIFDisplayTRC(int X,int Y,int W,int H,SDIF::Stream* s)
:SDIFDisplay(X,Y,W,H,s) {
	mColorMapping = new SDIFDisplayColors(0.3);
}

void SDIFDisplayTRC::ValuesAtTime(double *freq,double* amp,double time,
	bool quick)
{
	static int k=0;
	int i,j;
	k++;
	
	MoveIteratorTo(time);
	
	int sdifIds[256];
	int nSdifIds = 0;
	for (i=0;i<256;i++) {
		sdifIds[i]=-1;
	}
	for (i=0;i<256;i++) {
		freq[i]=0;
		amp[i]=0;
	}

	SDIF::Frame* left = *mFrameIterator;
	
	SDIF::ConcreteMatrix<float>* leftm = 
		dynamic_cast< SDIF::ConcreteMatrix<float>* >(left->GetMatrixAt(0));

	/*
	if (quick) {
		for (i=k&1;i<leftm->RowCount();i+=2) {
			sdif_int32 id;
			float f,a;
			leftm->Get(0,i,id);
			leftm->Get(1,i,f);
			leftm->Get(2,i,a);
			freq[id]=f;
			amp[id]=a;
		}

		return;
	}
	*/

	SDIF::Frame* right = 0;
	
	{
		SDIF::Stream::FrameIterator next = mFrameIterator;
		next++;
	
		if (next!=mStream->End())
		{
			right	= *(next);
		}
	}
	
	double wr = 0;
	if (right) {
		wr = (time - left->Time())/(right->Time() - left->Time());
	}
	double wl = 1.-wr;

	CLAM_ASSERT(leftm,"missing matrix");
	
	for (i=0;i<leftm->Rows();i++) {
		CLAM::TInt32 id;
		float f,a;
		id = CLAM::TInt32(leftm->GetValue(i,0));
		f = leftm->GetValue(i,1);
		a = leftm->GetValue(i,2);
		sdifIds[nSdifIds]=id;
		id = nSdifIds;
		nSdifIds++;
		freq[id]=f;
		amp[id]=a*wl;
	}

	if (right)
	{
		SDIF::ConcreteMatrix<float>* rightm =
				dynamic_cast< SDIF::ConcreteMatrix<float>* >(right->GetMatrixAt(0));

		CLAM_ASSERT(rightm,"missing matrix");

		for (i=0;i<rightm->Rows();i++) {
			CLAM::TInt32 id;
			float f,a;
			id = CLAM::TInt32(rightm->GetValue(i,0));
			f = rightm->GetValue(i,1);
			a = rightm->GetValue(i,2);
			for (j=0;j<nSdifIds;j++)
			{
				if (sdifIds[j]==id) {
					break;
				}
			}
			if (j==nSdifIds) {
				sdifIds[nSdifIds++]=id;
			}
			id = j;		
			if (freq[id] && f!=0.) freq[id]=freq[id]*wl+f*wr;
			else freq[id]=f;
			amp[id]+=a*wr;
		}
	}
}

void SDIFDisplayTRC::draw(void)
{
	fl_color(FL_BLACK);
	fl_rectf(x(),y(),w(),h());

	mFrameIterator = mStream->Begin();

	double t= HValueOffset();
	double dt= DHValue();

	double foffset = VValueOffset();
	double df = DVValue();
	
	int i;
	int cx=x();

	fl_push_clip(x(),y(),w(),h());

	double freq[256];
	double amp[256];

	fl_color(FL_WHITE);

	int prevy[256];
	for (i=0;i<256;i++) {
		prevy[i]=0x10000000;
	}
	
	for (cx=x();cx<x()+w();cx++) {
		ValuesAtTime(freq,amp,t,mQuick);
		for (i=0;i<256;i++) {
			if (amp[i]) {
				fl_color(mColorMapping->Get(amp[i]));
				
				int cy = y()+h()-int((freq[i]-foffset)/df);
				if (prevy[i]!=0x10000000) {
					fl_yxline(cx-1,cy,prevy[i]);
				}else{
					fl_yxline(cx,cy,cy);
				}
				prevy[i]=cy;
			}else{
				prevy[i]=0x10000000;
			}
		}
		t+=dt;
	}
	fl_pop_clip();	
}
